@@ -169,14 +169,14 @@ class Emitter(object):
         self.prefixed_colon = self.colon if prefix_colon is None else prefix_colon + self.colon
         # single entry mappings in flow sequence
         self.brace_single_entry_mapping_in_flow_sequence = (
-            brace_single_entry_mapping_in_flow_sequence
-        )  # NOQA
+            brace_single_entry_mapping_in_flow_sequence  # NOQA
+        )
 
         # Formatting details.
         self.canonical = canonical
         self.allow_unicode = allow_unicode
         # set to False to get "\Uxxxxxxxx" for non-basic unicode like emojis
-        self.unicode_supplementary = sys.maxunicode > 0xffff
+        self.unicode_supplementary = sys.maxunicode > 0xFFFF
         self.sequence_dash_offset = block_seq_indent if block_seq_indent else 0
         self.top_level_colon_align = top_level_colon_align
         self.best_sequence_indent = 2
@@ -410,8 +410,11 @@ class Emitter(object):
                 and self.sequence_context
             ):
                 self.sequence_context = False
-            if root and isinstance(self.event, ScalarEvent) \
-               and not self.scalar_after_indicator:
+            if (
+                root
+                and isinstance(self.event, ScalarEvent)
+                and not self.scalar_after_indicator
+            ):
                 self.write_indent()
             self.process_tag()
             if isinstance(self.event, ScalarEvent):
@@ -766,7 +769,8 @@ class Emitter(object):
             or (isinstance(self.event, MappingStartEvent) and self.event.flow_style is True)
             or (
                 isinstance(self.event, ScalarEvent)
-                and not self.analysis.empty
+                # if there is an explicit style for an empty string, it is a simple key
+                and not (self.analysis.empty and self.style and self.style not in '\'"')
                 and not self.analysis.multiline
             )
             or self.check_empty_sequence()
@@ -917,14 +921,14 @@ class Emitter(object):
         start = end = 0
         if prefix[0] == u'!':
             end = 1
+        ch_set = u"-;/?:@&=+$,_.~*'()[]"
+        if self.dumper:
+            version = getattr(self.dumper, 'version', (1, 2))
+            if version is None or version >= (1, 2):
+                ch_set += u'#'
         while end < len(prefix):
             ch = prefix[end]
-            if (
-                u'0' <= ch <= u'9'
-                or u'A' <= ch <= u'Z'
-                or u'a' <= ch <= u'z'
-                or ch in u"-;/?!:@&=+$,_.~*'()[]"
-            ):
+            if u'0' <= ch <= u'9' or u'A' <= ch <= u'Z' or u'a' <= ch <= u'z' or ch in ch_set:
                 end += 1
             else:
                 if start < end:
